@@ -10,11 +10,12 @@ print.gcrq <-function(x, digits = max(3, getOption("digits") - 4), ...){
       if(length(x$info.smooth)>0){
          cat("\nEquivalent Degrees of Freedom:\n")
          if(rownames(x$edf.j)[1]=="Xlin") rownames(x$edf.j)[1]<-"unpenalized"
+         attr(x$edf.j,"df.nc")<- NULL
          print(round(x$edf.j, digits))
       }
       all.sic <- log(x$rho/n) + log(n) * colSums(as.matrix(x$edf.j))/(2*n)
       sic<- sum(all.sic) #sum(log(x$rho/n)) +log(n)*sum(x$edf.j)/(2*n)
-      cat("\nOverall check fn =", round(sum(x$rho),digits), "  SIC =", round(sic,digits), " (on edf =",paste(round(sum(x$edf.j),digits),")",sep="") ,"\n")
+      cat("\nOverall check =", round(sum(x$rho),digits), "  SIC =", round(sic,digits), " (on edf =",paste(round(sum(x$edf.j),digits-1),")",sep="") ,"\n")
 }
 
 
