@@ -1,7 +1,7 @@
 ps <-
 function(..., lambda=-1, d=3, by=NULL, ndx=NULL, deg=3, knots=NULL, 
     monotone=0, concave=0, var.pen=NULL, pen.matrix=NULL, dropc=TRUE, center=TRUE,
-    K=2, ridge=FALSE, decompose=FALSE){
+    K=2, decompose=FALSE, constr.fit=TRUE, shared.pen=FALSE){
 #arguments in ... such as 'a=3' are ignored
 #dropc, if TRUE the first column of the basis is dropped..
 #decompose: if TRUE the decomposition of the B-spline is employed. Notice the new basis becomes
@@ -35,14 +35,23 @@ function(..., lambda=-1, d=3, by=NULL, ndx=NULL, deg=3, knots=NULL,
     nomi<-nomi[id]    
     names(vars)<-nomi
     
-    p<-length(nomi)
-    r<-matrix(unlist(vars), ncol=p, byrow = FALSE)    
+
+    if(is.factor(vars[[1]]) || is.matrix(vars[[1]]) || length(nomi)>1) stop("ps() accepts single numerical terms only")
+    #if(length(nomi)>1) stop(" ps() accepts single terms only") #: numeric/factor vector or matrix")
+    
+    p<-1
+    if(is.matrix(vars[[1]])) p<- ncol(vars[[1]])
+    r<-matrix(unlist(vars), ncol=p, byrow = FALSE, dimnames = list(NULL, colnames(vars[[1]])))    
+    
     #da mgcv::s
     #vars <- as.list(substitute(list(...)))[-1]
     #p<-length(vars)
     #by.var <- deparse(substitute(by), backtick = TRUE, width.cutoff = 500)
     #term <- deparse(vars[[1]], backtick = TRUE, width.cutoff = 500)
-    r<- if(!is.null(by)) cbind(r,by) else cbind(r,1)
+    #browser()
+    r<- if(!is.null(by)) cbind(r,by) else cbind(r) #cbind(r,1)
+    #r<- apply(r,2, function(.x)as.numeric(as.character(as.factor(.x)))) #non funziona se la prima colonna non ha numeri..
+    #r<- apply(r,2, function(.x)as.numeric(as.factor(.x)))
     attr(r,"penMatrix")<-pen.matrix
     attr(r,"nodi")<-knots
     attr(r,"ndx")<-ndx
@@ -50,23 +59,28 @@ function(..., lambda=-1, d=3, by=NULL, ndx=NULL, deg=3, knots=NULL,
     attr(r,"pdiff")<-d
     attr(r,"monot")<-monotone #isTRUE(monotone)
     attr(r,"conc")<-concave
+    attr(r,"constr.fit")<-constr.fit
     attr(r,"lambda")<-lambda
-    attr(r,"nomeX")<- nomi#deparse(substitute(x))
+    attr(r,"nomeX")<- (nomi[[1]])#deparse(substitute(x))
     attr(r,"var.pen")<-var.pen
     attr(r,"K")<-K
-    attr(r,"ridge") <-ridge
+    attr(r,"ridge") <-if(is.matrix(vars[[1]]) || is.factor(vars[[1]])) TRUE else FALSE
     attr(r,"nomeBy")<-deparse(substitute(by), backtick = TRUE, width.cutoff = 500)
     attr(r,"levelsBy")<-levels(by)
     attr(r,"dimSmooth")<-p
     attr(r,"decom")<-decompose
-    if(monotone!=0 && decompose) stop("'decom=TRUE' is incompatible with monotonicity restrictions")
+    attr(r,"shared.pen")<-shared.pen
+    if(monotone!=0 && decompose) stop("'decom=TRUE' is incompatible with monotonicity restrictions") #se i vincoli sono sui fitted e' OK..
     if(concave!=0 && decompose) stop("'decom=TRUE' is incompatible with concavity restrictions")
     if(decompose) dropc<-FALSE
-    colnames(r)<-c(nomi, "by")
+    
+    if(!is.null(by)) colnames(r)<-c(nomi, "by") else nomi
+    #colnames(r)<-c(nomi, "by")
     #if(!is.null(by)) colnames(r)<-c(deparse(substitute(x)), deparse(substitute(by)))
     attr(r,"dropc")<-dropc
     attr(r,"center")<-center
     if(dropc && decompose) stop("'decom=TRUE' is incompatible with 'dropc=TRUE' ")
+	#browser()
 	r
 }
 
